@@ -17,6 +17,8 @@ from werkzeug.datastructures import ImmutableMultiDict
 from trexapi.forms.user_api_forms import UserRegistrationForm
 from trexapi.conf import APPLICATION_NAME, APPLICATION_BASE_URL
 from trexmail.email_helper import trigger_send_email
+from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet
+from trexmodel.models.datastore.customer_models import Customer
 
 user_api_bp = Blueprint('user_api_bp', __name__,
                                  template_folder='templates',
@@ -198,6 +200,109 @@ def verify_mobile_phone_account():
     else:
         logger.warn('verify_mobile_phone_account: user verify input is invalid')
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)        
+
+@user_api_bp.route('/register-as-customer', methods=['POST'])
+def register_user_as_customer():
+    user_data_in_json           = request.get_json()
+    reference_code              = user_data_in_json.get('reference_code')
+    merchant_reference_code     = user_data_in_json.get('merchant_reference_code')
+    
+    logger.debug('register_user_as_customer: user_data_in_json=%s', user_data_in_json)
+    
+    try:
+        if is_not_empty(reference_code):
+            logger.debug('customer registration input is valid')
+            db_client = create_db_client(caller_info="register_user_as_customer")
+            
+            created_customer        = None
+            existing_user_acct      = None
+            is_email_used           = False
+            is_mobile_phone_used    = False
+            outlet_key              = request.headers.get('x-outlet-key')
+            merchant_act_key        = request.headers.get('x-merchant-acct-key')
+            merchant_acct           = None
+            
+            with db_client.context():
+                merchant_acct       = MerchantAcct.fetch(merchant_act_key)
+                outlet              = Outlet.fetch(outlet_key)
+                
+                    
+                if merchant_acct and outlet:
+                        
+                    logger.debug('merchant_acct.key_in_str=%s', merchant_acct.key_in_str)
+                    logger.debug('outlet.merchant_acct_key=%s', outlet.merchant_acct_key)
+                    
+                    
+                    if merchant_acct.key_in_str == outlet.merchant_acct_key: 
+                        logger.debug('Valid granted outlet key for merchant acct')
+                        
+                        created_customer = Customer.get_by_reference_code(reference_code)
+                         
+                        if created_customer is None:
+                            existing_user_acct  = User.get_by_reference_code(reference_code)
+                            
+                            email           = existing_user_acct.email
+                            mobile_phone    = existing_user_acct.mobile_phone
+                            
+                            logger.debug('email=%s', email)
+                            logger.debug('mobile_phone=%s', mobile_phone)
+                            
+                            checking_customer = Customer.get_by_email(email) 
+                            
+                            if checking_customer:
+                                is_email_used = True
+                            else:
+                                if is_not_empty(mobile_phone):
+                                    checking_customer = Customer.get_by_mobile_phone(mobile_phone)
+                                    if checking_customer:
+                                        is_mobile_phone_used = True
+                            
+                            logger.debug('is_email_used=%s', is_email_used)
+                            logger.debug('is_mobile_phone_used=%s', is_mobile_phone_used)
+                            
+                            if is_email_used == False and is_mobile_phone_used == False:
+                            
+                                created_customer = Customer.create_from_user(merchant_acct, outlet, existing_user_acct, merchant_reference_code=merchant_reference_code)
+                        
+                        logger.debug('created_customer=%s', created_customer)
+                    else:
+                        logger.warn('Invalid granted outlet key')
+                else:
+                    logger.warn('Invalid granted outlet key or merchant account id')
+                
+                if created_customer:
+                    
+                    response_data = {
+                                    'customer_key'              : created_customer.key_in_str,
+                                    'registered_datetime'       : created_customer.registered_datetime.strftime("%d-%m-%Y %H:%M:%S"),
+                                    'merchant_reference_code'   : created_customer.merchant_reference_code,
+                                    'reference_code'            : created_customer.reference_code,
+                                    'merchant_account_key'      : merchant_act_key,
+                                    'company_name'              : merchant_acct.name,
+                                    'outlet_key'                : outlet_key,  
+                                    }
+                    
+                    logger.debug('response_data=%s', response_data)
+                    
+                    return create_rest_message(status_code=StatusCode.OK, **response_data)
+                    
+                else:
+                    if is_email_used==True:
+                        return create_rest_message('Email have been taken', status_code=StatusCode.BAD_REQUEST)
+                    
+                    elif is_mobile_phone_used==True:
+                        return create_rest_message('Mobile phone have been taken', status_code=StatusCode.BAD_REQUEST)
+                    else:
+                        return create_rest_message('Failed to register customer', status_code=StatusCode.BAD_REQUEST)
+            
+        else:
+            logger.warn('customer registration input is invalid')
+            
+            return create_rest_message("Missing register customer input data", status_code=StatusCode.BAD_REQUEST)
+    except:
+        logger.error('Fail to register customer due to %s', get_tracelog())
+        
+        return create_rest_message(status_code=StatusCode.BAD_REQUEST)
     
 @user_api_bp.route('/<reference_code>', methods=['GET'])
 def read_user_acct(reference_code):
@@ -234,6 +339,7 @@ def read_user_acct(reference_code):
                                        mobile_phone_vc_expiry_datetime      = mobile_phone_vc_expiry_datetime,
                                        )
         else:
+            logger.debug('user account is not found')
             return create_rest_message(status_code=StatusCode.BAD_REQUEST)
             
     else:
@@ -351,6 +457,7 @@ def request_reset_password_post():
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)                 
 
 
+
 def send_email_verification_code_email(user_acct):
     message = '''
                 Hi {name},
